import typer
import json
from typing import Annotated
from rich.console import Console
from nixprstatus.pr import pr_merge_status
from nixprstatus.pr import commits_since
from nixprstatus.watchlist import Watchlist
from nixprstatus.watchlist import OutputFormat

app = typer.Typer(rich_markup_mode=None)
watchlist_app = typer.Typer()
app.add_typer(watchlist_app, name="watchlist", help="Manage watchlist.")

DEFAULT_HEADERS = {
    "Accept": "application/vnd.github.text+json",
}

BRANCHES = ["nixos-unstable-small", "nixos-unstable", "nixos-24.05"]


@app.command()
def pr(
    pr: int,
    branches: Annotated[
        list[str] | None, typer.Option(help="Check specific branch")
    ] = None,
):
    """Get merge status of pull request."""
    console = Console()

    if branches:
        status = pr_merge_status(pr, branches)
    else:
        status = pr_merge_status(pr)

    console.print(f"{status.title}\n", highlight=False)
    merged = ":white_check_mark: merged" if status.merged else ":x: merged"
    console.print(merged, highlight=False)

    for branch in status.branches:
        output = (
            f":white_check_mark: {branch}"
            if status.branches[branch]
            else f":x: {branch}"
        )
        console.print(output, highlight=False)


@app.command()
def since(
    commit_sha: str,
    target: str = "nixos-unstable",
    waybar_format: Annotated[
        bool, typer.Option(help="Output in format expected by waybar")
    ] = False,
):
    """
    Return the count of commits that has happened between the two refs.
    """
    count = commits_since(target, commit_sha)

    if waybar_format:
        output = {"text": str(count), "tooltip": f"{target}: {count}"}
        typer.echo(json.dumps(output))
        return
    typer.echo(count)

@watchlist_app.command()
def list(watchlist: str|None = None, format: OutputFormat = OutputFormat.CONSOLE):
    """List PRs in watchlist."""
    wl = Watchlist.from_file()
    wl.print(format=format)

@watchlist_app.command()
def add(pr: int):
    """Add PR to watchlist."""
    wl = Watchlist.from_file()
    wl.add_pr(pr)
    wl.to_file()
    info = wl.pr(pr)
    print(f"Added #{info.pr}: {info.title} to watchlist.")

@watchlist_app.command()
def remove(pr: int):
    """Remove PR from watchlist."""
    wl = Watchlist.from_file()
    wl.remove(pr)
    wl.to_file()
    print(f"Removed #{pr} from watchlist.")

def main():
    app()


if __name__ == "__main__":
    main()
